Package: esetVis
Type: Package
Title: Visualizations of expressionSet Bioconductor object
Version: 1.39.0
Date: 2023-12-15
Author: Laure Cougnaud <laure.cougnaud@openanalytics.eu>
Maintainer: Laure Cougnaud <laure.cougnaud@openanalytics.eu>
Description: Utility functions for visualization of expressionSet (or
        SummarizedExperiment) Bioconductor object, including spectral
        map, tsne and linear discriminant analysis. Static plot via the
        ggplot2 package or interactive via the ggvis or rbokeh packages
        are available.
Imports: mpm, hexbin, Rtsne, MLP, grid, Biobase, MASS, stats, utils,
        grDevices, methods
Suggests: ggplot2, ggvis, plotly, ggrepel, knitr, rmarkdown, ALL,
        hgu95av2.db, AnnotationDbi, pander, SummarizedExperiment, GO.db
biocViews: Visualization, DataRepresentation, DimensionReduction,
        PrincipalComponent, Pathways
VignetteBuilder: knitr
License: GPL-3
NeedsCompilation: no
RoxygenNote: 7.2.3
git_url: https://git.bioconductor.org/packages/esetVis
git_branch: devel
git_last_commit: 268e44c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 22:35:16 UTC; biocbuild
