# Generated by roxygen2: do not edit by hand

export(esetLda)
export(esetPlot)
export(esetPlotInteractive)
export(esetPlotWrapper)
export(esetSpectralMap)
export(esetTsne)
export(getGeneSetsForPlot)
export(ggplotEsetPlot)
export(ggvisEsetPlot)
export(plotEset)
export(plotlyEsetPlot)
exportClasses(esetPlot)
exportClasses(esetPlotInteractive)
exportClasses(ggplotEsetPlot)
exportClasses(ggvisEsetPlot)
exportClasses(plotlyEsetPlot)
import(Biobase)
import(mpm)
importFrom(MASS,lda)
importFrom(MLP,getGeneSets)
importFrom(Rtsne,Rtsne)
importFrom(grDevices,colorRamp)
importFrom(grDevices,colorRampPalette)
importFrom(grid,unit)
importFrom(hexbin,hexbin)
importFrom(methods,callNextMethod)
importFrom(methods,new)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,setNames)
importFrom(utils,capture.output)
importFrom(utils,getFromNamespace)
importFrom(utils,packageVersion)
